/*
 * =========================================================================
 * ||              TEST SKETCH FOR THE  SD CARD MODULE                    ||
 * =========================================================================
 *
 * Copyright (c) 2013, C&D Elettronica <info@cedelettronica.com>
 * htpp://www.cedelettronica.com
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* 
 * INFO (ENGLISH)
 *
 * It is assumed that you are already quite comfortable with developing for Arduino, like using
 * the IDE, uploading a sketch, blinking an LED, using the Serial communication.
 * If you are a complete beginner, you might want to have a look at
 * http://arduino.cc/en/Guide/HomePage (also look at the Troubleshooting and Environment sections)
 * and possibly at one of the very many tutorials about Arduino available on the net.
 *
 * This sketch tests:
 * ==================
 * + The SD card
 *
 * >>> See also schematic on the website to see how to connect the board. <<<
 *
 * 
 * To test the SD card:
 * ====================
 * + With the power supply disconnected, plug in the SD card.
 *   You might want to create some files/folders on the SD from your PC, to check that they are
 *   correctly read by the board.
 * + Connect the power supply and open a serial terminal.
 * + Check that the SD card has been correctly initialized (see the message printed over serial at startup)
 * + Enter any character : the list of files on SD will be printed.
 *
 *
 * Troubleshooting:
 * ================
 * + The SD card does not work.
 * > Check that it has been formatted as FAT.
 *
 * + The SD card does not work/I don't see files in the SD.
 * > The Arduino library for SD only accepts "8.3" files: the filename must have at most 8 characters,
 *   and at most 3 characters for the extension. Also, avoiding dots in the filename might help.
 */

/* 
 * INFORMAZIONI (ITALIANO)
 *
 * Questa guida parte dal presupposto che hai gia' dimestichezza con lo sviluppo per Arduino, ad
 * esempio usare l'IDE, caricare uno sketch, far lampeggiare un LED, usare la comunicazione seriale.
 * Se sei un principiante assoluto, dovresti prima dare un'occhiata a 
 * http://arduino.cc/en/Guide/HomePage (vedi anche le sezioni Troubleshooting e Environment)
 * e magari anche a uno dei molti moltissimi tutorial su Arduino che si trovano in rete.
 *
 * Questo sketch serve per testare:
 * ================================
 * + La SD card
 *
 * >>> Vedi anche il foto-tutorial sul sito, per vedere come collegare la board. <<<
 *
 * Per testare la SD card:
 * =======================
 * + Con la board non alimentata, inserisci la SD card.
 *   Puoi anche creare dei file/cartelle sulla SD dal tuo PC, per verificare poi che la board li legga correttamente.
 * + Alimenta la board e apri il terminale seriale.
 * + Controlla che la SD card sia stata inizializzata correttamente (guarda il messaggi mostrato nel terminal serial all'avvio).
 * + Invia un carattere qualsiasi : verra' stampata la lista dei file sulla SD.
 *
 * Risoluzione dei problemi:
 * =========================
 * + La SD card non funziona.
 * > Controlla che la SD sia formattata come FAT.
 *
 * + La SD card non funziona/non vedo alcun file sulla SD.
 * > La libreria Arduino per SD accetta solo nomi in formato "8.3": il nome del file deve contenere
 *   al piu' 8 caratteri, e al piu' 3 caratteri per l'estensione. Inoltre, evitare di mettere punti
 *   nel nome potrebbe aiutare.
 *
 */



#include <SD.h>
#include <Ethernet.h>
#include <SPI.h>

bool sdOk;

const byte SD_CS_PIN = 43;
const byte LED_PIN = 13;

void setup()
{
    Serial.begin(9600); // SETTING: the baudrate you will use in the serial terminal
                        //          il baudrate che usi nel terminale della porta seriale
   
    
    // If you use the SD, its library will handle this itself.
    pinMode(SD_CS_PIN, OUTPUT);
    digitalWrite(SD_CS_PIN, HIGH);
    
    Serial.print(F("Initializing SD card... "));
    sdOk = SD.begin(SD_CS_PIN);
    if (sdOk) {
        Serial.println(F("initialization done."));
    } else {
        Serial.println(F("initialization failed!"));
    }
  
    pinMode(LED_PIN,OUTPUT); // set to output to blink the LED
}

void loop()
{
    static bool led = true;
    static unsigned long time = 0;
   
    if(millis() - time >= 1000){
        // blink LED at every 1000ms
        time = millis();
        digitalWrite(LED_PIN,led);
        led = !led;
    }
    
    // check if the user has sent something over the serial
    if(Serial.available()) {
        //char c = Serial.read();
            printDir();
            while(Serial.read() >= 0); // empty the serial buffer  
    }
}    

void printDir() {
    Serial.println(F("SD content:"));
    if(!sdOk) {
        Serial.println(F("\tSD is not OK"));
    } else {
      
        File root = SD.open("/");
        
        if(!root) {
            Serial.println(F("\t&lt;Error while reading the SD card's root&gt;"));
        } else {
            bool empty = true;
            for(;;) {
                File entry = root.openNextFile();
                if (! entry) { // no more files
                    if(empty) {
                        Serial.println(F("\t&lt;SD is empty&gt;"));
                    }
                    break;
                }
                empty = false;
                
                Serial.print('\t');
                Serial.println(entry.name());
                entry.close();
            }
        }
    }
}
